/**
  ******************************************************************************
  * @file	rtos_bluetooth.c
  * @version 	1.0
  * @brief	Handles the Bluetooth communication
 ******************************************************************************
  * @attention
  *
  * TODO
  *
  ******************************************************************************
  */

#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <em_device.h>

#ifdef APP_NCP
#include "rtosncp_gecko.h"
#else
#include "rtos_gecko.h"
#endif
#include "gecko_configuration.h"
#include "rtos_bluetooth.h"

#ifdef CONFIGURATION_HEADER
#include CONFIGURATION_HEADER
#endif // CONFIGURATION_HEADER

void bluetoothUpdate();
volatile struct gecko_cmd_packet*   bluetoothEvent;
OS_MUTEX                            bluetoothMutex;

volatile static uint32_t            commandHeader;
volatile static void*               commandData;
volatile static gecko_cmd_handler   commandHandlerFunc = NULL;

/* Bluetooth task */
#ifndef BLUETOOTH_STACK_SIZE
#define BLUETOOTH_STACK_SIZE        (1500 / sizeof(uint32_t))
#endif
static void                         bluetoothTask(void);
static OS_TASK                      bluetoothTaskTCB;
static OS_STACKPTR uint32_t         bluetoothTaskStack[BLUETOOTH_STACK_SIZE];

/* Link-Layer task */
#ifndef LINKLAYER_STACK_SIZE
#define LINKLAYER_STACK_SIZE        (1000 / sizeof(uint32_t))
#endif 
static void                         linkLayerTask(void);
static OS_TASK                      linkLayerTaskTCB;
static OS_STACKPTR uint32_t         linkLayerTaskStack[LINKLAYER_STACK_SIZE];

/* Bluetooth Timing */
#define RTOS_TICK_HZ                1024
#define BLUETOOTH_TICK_HZ           32768
#define BLUETOOTH_TO_RTOS_TICK      (BLUETOOTH_TICK_HZ / RTOS_TICK_HZ)

static volatile wakeupCallback      wakeupCB = NULL;

OS_EVENT                            bluetoothOSEvent;

/* Set the task to post semaphore */
void bluetoothSetWakeupCallback(wakeupCallback cb)
{
    wakeupCB = cb;
}

void bluetoothStart(OS_PRIO linkLayerPriority,
                    OS_PRIO stackPriority,
                    bluetoothStackInitFunc initializeBluetoothStack)
{
    OS_EVENT_CreateEx(&bluetoothOSEvent, OS_EVENT_RESET_MODE_SEMIAUTO | OS_EVENT_MASK_MODE_OR_LOGIC);
    OS_MUTEX_Create(&bluetoothMutex);

    /* Initialize Bluetooth stack */
    OS_ASSERT((initializeBluetoothStack != NULL), 1);
    initializeBluetoothStack();

    /* Create task for Bluetooth Host Stack */
    OS_TASK_CREATE(&bluetoothTaskTCB, 
                   "Bluetooth Host Task",
                   stackPriority,
                   bluetoothTask,
                   bluetoothTaskStack);

    /* Create task for Link-Layer */
    OS_TASK_CREATE(&linkLayerTaskTCB,
                   "LinkLayer Task",
                   linkLayerPriority,
                   linkLayerTask,
                   linkLayerTaskStack);
}

/* 
 * This callback is called from interrupt context (Kernel Aware)
 * Sets Event to trigger Link-Layer task
 */
void bluetoothLinkLayerCallback(void)
{
    OS_EVENT_SetMask(&bluetoothOSEvent, BLUETOOTH_EVENT_BITMASK_LL);
}

/*
 * This callback is called from Bluetooth stack
 * Called from Kernel aware interrupt context (RTCC interrupt) and from Bluetooth task
 * Sets Event to trigger running Bluetooth task
 */
void bluetoothUpdate(void)
{
    OS_EVENT_SetMask(&bluetoothOSEvent, BLUETOOTH_EVENT_BITMASK_STACK);
}

/* Bluetooth task, it waits for events from bluetooth and handles them */
void bluetoothTask(void)
{
    uint32_t timeout;
    OS_TASKEVENT eventBitMask = BLUETOOTH_EVENT_BITMASK_EVT_HANDLED;

    /* Bluetooth Task Loop */
    while(true)
    {
        /* Command needs to be sent to Bluetooth stack */
        if(eventBitMask & BLUETOOTH_EVENT_BITMASK_CMD_WAITING)
        {
            printf("rtos_ble: 1\r\n");
            uint32_t header = commandHeader;
            gecko_cmd_handler cmdHandler = commandHandlerFunc;
            sli_bt_cmd_handler_delegate(header, cmdHandler, (void*)commandData);
            commandHandlerFunc = NULL;
            eventBitMask &= ~BLUETOOTH_EVENT_BITMASK_CMD_WAITING;
            OS_EVENT_SetMask(&bluetoothOSEvent, BLUETOOTH_EVENT_BITMASK_RSP_WAITING);
        }
        /* Bluetooth stack needs updating and evt can be used */
        if(gecko_event_pending() && (eventBitMask & BLUETOOTH_EVENT_BITMASK_EVT_HANDLED))
        {
            printf("rtos_ble: 2\r\n");
            bluetoothEvent = gecko_wait_event();
            OS_ASSERT(bluetoothEvent, 1);
            OS_EVENT_SetMask(&bluetoothOSEvent, BLUETOOTH_EVENT_BITMASK_EVT_WAITING);
            eventBitMask &= ~BLUETOOTH_EVENT_BITMASK_EVT_HANDLED;
            if(wakeupCB != NULL)
            {
                wakeupCB();
            }
        }
        /*
         * Ask from Bluetooth stack how long we can sleep
         * UINT32_MAX = sleep indefinitely
         * 0 = cannot sleep, stack needs update and we need to check
         * if evt is handled that we can actually update it
         */
        timeout = gecko_can_sleep_ticks();
        if(timeout == 0 && (eventBitMask & BLUETOOTH_EVENT_BITMASK_EVT_HANDLED))
        {
            continue;
        }
        if(timeout == UINT32_MAX)
        {
            printf("rtos_ble: 3\r\n");
            eventBitMask |= OS_EVENT_GetMaskBlocked(&bluetoothOSEvent, BLUETOOTH_EVENT_BITMASK_STACK + BLUETOOTH_EVENT_BITMASK_EVT_HANDLED + BLUETOOTH_EVENT_BITMASK_CMD_WAITING);
        }
        else
        {
            printf("rtos_ble: 4\r\n");
            /* Round up to RTOS ticks */
            timeout = (timeout + BLUETOOTH_TO_RTOS_TICK - 1) / BLUETOOTH_TO_RTOS_TICK;
            eventBitMask |= OS_EVENT_GetMaskTimed(&bluetoothOSEvent, BLUETOOTH_EVENT_BITMASK_STACK + BLUETOOTH_EVENT_BITMASK_EVT_HANDLED + BLUETOOTH_EVENT_BITMASK_CMD_WAITING, timeout);
        }
    }
}

void linkLayerTask(void)
{
    /* Link-Layer Task Loop */
    while(true)
    {
        printf("rtos_ble: 5\r\n");
        OS_EVENT_GetMaskBlocked(&bluetoothOSEvent, BLUETOOTH_EVENT_BITMASK_LL);
        gecko_priority_handle();
    }
}

/* Hooks for API. Called from tasks using BGAPI */
void sli_bt_cmd_handler_rtos_delegate(uint32_t header, gecko_cmd_handler handler, const void* payload)
{
    commandHeader = header;
    commandHandlerFunc = handler;
    commandData = (void*)payload;

    /* Command structure is filled, notify the stack */
    OS_EVENT_SetMask(&bluetoothOSEvent, BLUETOOTH_EVENT_BITMASK_CMD_WAITING);

    /* Wait for response */
    OS_EVENT_GetMaskBlocked(&bluetoothOSEvent, BLUETOOTH_EVENT_BITMASK_RSP_WAITING);
}

/* Mutex functions for using Bluetooth from multiple tasks */
void bluetoothLockMutex(OS_U8* mutexStatus)
{
    *mutexStatus = OS_MUTEX_Lock(&bluetoothMutex);
}

void bluetoothUnlockMutex(void)
{
    OS_MUTEX_Unlock(&bluetoothMutex);
}
